# -*- coding: utf-8 -*-

"""
Selenium PageModel library
~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2017 by Peter Salas.
:license: Apache 2.0, see LICENSE for more details.
"""

__title__ = 'seleniumpm'
__version__ = '2.4.2'
__build__ = 0x021000
__author__ = 'Peter Salas'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2017 Peter Salas'



from seleniumpm.webelements.clickable import Clickable


class Widget(Clickable):
    def __init__(self, driver, locator):
        super(Clickable, self).__init__(driver, locator)

    def validate(self):
        raise NotImplementedError



from selenium.webdriver.support.select import Select
from seleniumpm.webelements.clickable import Clickable

from seleniumpm.webelements.element import Element


class Dropdown(Clickable):
    def __init__(self, driver, locator):
        super(Dropdown, self).__init__(driver, locator)
        self.select = None

    def instantiateSelect(self):
        if self.select == None:
            self.select = Select(self.driver.find_element(self.locator.by, self.locator.value))

    def count_options(self):
        # TODO - Need to implement a way to get and modify the original locator to append '/option' (Assuming xpath)
        raise NotImplementedError

    def select_by_visible_text(self, txt):
        self.instantiateSelect()
        self.select.select_by_visible_text(txt)
        return self

    def select_by_index(self, index):
        self.instantiateSelect()
        self.select.select_by_index(index)
        return self



from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC


class Element(object):
    def __init__(self, driver, locator):
        self.driver = driver
        self.locator = locator

    def get_webelement(self):
        return self.driver.find_element(self.locator.by, self.locator.value)

    def get_webelements(self):
        return self.driver.find_elements(self.locator.by, self.locator.value)

    def get_text(self):
        return self.get_webelement().text

    def get_attribute(self, name):
        self.get_webelement().get_attribute(name)

    def is_displayed(self):
        return self.get_webelement().is_displayed()

    def is_enabled(self):
        return self.get_webelement().is_enabled()

    def is_selected(self):
        return self.get_webelement().is_selected()

    def is_present(self, timeout=10):
        try:
            self.wait_for_present(timeout)
            return True
        except:
            return False

    def is_visible(self, timeout=10):
        try:
            self.wait_for_visible(timeout)
            return True
        except:
            return False

    def is_present_and_visible(self, timeout=10):
        try:
            self.is_present(timeout)
            self.is_visible(0)
            return True
        except:
            return False

    def wait_for_present(self, timeout=10):
        WebDriverWait(self.driver, timeout).until(
            EC.presence_of_element_located((self.locator.by, self.locator.value)))
        return self

    def wait_for_visible(self, timeout=10):
        WebDriverWait(self.driver, timeout).until(
            EC.visibility_of_element_located((self.locator.by, self.locator.value)))
        pass

    def wait_for_present_and_visible(self, timeout=10):
        self.wait_for_present(timeout)
        self.wait_for_visible(0)
        return self

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return NotImplemented

    def __ne__(self, other):
        if isinstance(other, self.__class__):
            return not self.__eq__(other)
        return NotImplemented

    def __hash__(self):
        return hash(tuple(sorted(self.__dict__.items())))



from seleniumpm.webelements.textelement import TextElement


class TextField(TextElement):
    def __init__(self, driver, locator):
        super(TextField, self).__init__(driver, locator)

    def send_keys(self, txt):
        self.driver.find_element(self.locator.by, self.locator.value).send_keys(txt)

    def type(self, txt):
        self.send_keys(txt)

    def clear(self):
        self.driver.find_element(self.locator.by, self.locator.value).clear()
        return self

    def submit(self):
        self.driver.find_element(self.locator.by, self.locator.value).submit()



from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from urlparse import urlparse
import re

url_regex = re.compile(
    r'^(?:http|ftp)s?://'  # http:// or https://
    r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
    r'localhost|'  # localhost...
    r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
    r'(?::\d+)?'  # optional port
    r'(?:/?|[/?]\S+)$', re.IGNORECASE)


class Webpage(object):
    """
    The Webpage class is intended to be the parent class for all Webpages. In principle, a Webpage is simply a
    construct that allows us to organize a set of WebElements on any given page; therefore, this class and other
    Selenium PageModel entities follows this paradigm.

    The first step to do is to write your own class that extends Webpage:

        class Google(Webpage):
            def __init__(self, driver, locator):
                super(Google, self).__init__(driver, locator)
                self.path = ""
                self.search_field = TextField(driver, Locator(By.NAME, 'q'))

            def wait_for_page_load(self, timeout=30):
                self.search_field.wait_for_present_and_visible(timeout=timeout)
                return self

            def validate(self, timeout=10):
                self.search_field.wait_for_present_and_visible(timeout=timeout)

    In the constructor, you do the following:

        1) Call the parent class's constructor
        2) You define 'path' attribute (default is "", so it didn't need to be specified in this example)
        3) You add a set of WebElements to your Webpage

    There are two additional methods that you should override from the parent:

        1) wait_for_page_load() - This is used for defining a set of WebElements that can be checked to determine if the
                                  Webpage was successfully loaded
        2) validate() - This is used for quick validation of a set of WebElements on a page (e.g.
                        registration_page.validate())
    """

    def __init__(self, driver, url=None):
        self.driver = driver
        self.path = ""
        # Check if a valid url
        if url and not url_regex.search(url.geturl() if isinstance(url, tuple) else url):
            raise AttributeError("Invalid url: '{}'".format(url))
        # Check if  url is None or has already been parsed
        if url == None or isinstance(url, tuple):
            self.url = url
        # Check if url is defined but has not been parsed
        elif url and not isinstance(url, tuple):
            self.url = urlparse(url)

    def open(self, url=None):
        """
        This method has two forms of operation:

        1) It will open whatever url that is passed into method
        2) It will use the url specified when the WebPage object was specified (Recommended)

        The latter method is the recommended approach.
        :param url: The url to open, but it is recommended that this be specified in constructor - Default: None
        :return:
        """
        if url:
            self.driver.get(url)
        elif self.url:
            url = "{}://{}:{}{}".format(
                self.url.scheme,
                self.url.hostname,
                self.url.port if self.url.port else (80 if self.url.scheme == "http" else 443),
                self.path)
            self.driver.get(url)
        else:
            raise AttributeError("Url is not defined!")
        return self

    def close(self):
        """Closes the browser
        """
        self.driver.close()

    def quit(self):
        """Quits the Selenium session
        """
        self.driver.quit()

    def get_title(self):
        """Returns the title of the WebPage
        """
        return self.driver.title

    def wait_for_title(self, title, timeout=10):
        """This could be used similar to a wait_for_page_load() if the page title can uniquely identify
        different pages or states of the page. Google Search works like this.
        :param title: The title to search for (case sensitive)
        :param timeout: The number of seconds to wait - Default: 10
        :raises TimeoutException: if the title does not appear within timeout period
        """
        WebDriverWait(driver=self.driver, timeout=timeout).until(EC.title_contains(title))
        return self

    def wait_for_page_load(self):
        """
        This method "waits for page load" by checking that all expected objects are both present and visible on the
        page. This is similar to validate() operation except that sometimes certain pages take a long time to load.
        Typically the threshold is 30sec, but this is configurable.
        :return: self if everything is successful
        :raises TimeoutException: if an element doesn't appear within timeout
        """
        raise NotImplementedError

    def validate(self):
        """
        The intention of validate is to make sure that an already loaded webpage contains these elements.
        :return:
        :raises TimeoutException: if an element doesn't appear within timeout
        """
        raise NotImplementedError



import copy

class Locator(object):

    def __init__(self, by, value):
        self.by = by
        self.value = value

    def append(self, relative_path):
        self.value += relative_path

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return NotImplemented

    def __ne__(self, other):
        if isinstance(other, self.__class__):
            return not self.__eq__(other)
        return NotImplemented

    def __hash__(self):
        return hash(tuple(sorted(self.__dict__.items())))

    def copy(self):
        return copy.deepcopy(self)



#!/usr/bin/env python

import os
import re
import sys

from codecs import open

# from distutils.core import setup
from setuptools import setup
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass into py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest

        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

packages = ['seleniumpm', 'seleniumpm.webelements', 'seleniumpm.examples']

requires = ['selenium~=2.53.6']
test_requirements = ['pytest>=2.8.0', 'pytest-httpbin==0.0.7', 'pytest-cov']

with open('seleniumpm/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

readme_file = 'README.rst'
with open(readme_file, 'r', 'utf-8') as f:
    readme = f.read()
with open('HISTORY.rst', 'r', 'utf-8') as f:
    history = f.read()

setup(
    name='seleniumpm',
    version=version,
    description='Selenium Pagemodel implementation for Python.',
    long_description=readme + "\n\n" + history,
    author='Peter Salas',
    author_email='selenium-pagemodel@googlegroups.com',
    url='https://github.com/gradeawarrior/python-seleniumpm',
    packages=packages,
    package_data={'': ['LICENSE', 'NOTICE'], 'seleniumpm': ['*.pem']},
    package_dir={'seleniumpm': 'seleniumpm'},
    include_package_data=True,
    install_requires=requires,
    license='Apache 2.0',
    keywords=['testing', 'seleniumpm', 'selenium', 'pagemodel', 'pageobjectmodel'],
    zip_safe=False,
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ),
    cmdclass={'test': PyTest},
    tests_require=test_requirements,
)



from seleniumpm.webelements.element import Element


class Clickable(Element):
    def __init__(self, driver, locator):
        super(Clickable, self).__init__(driver, locator)

    def click(self, checkVisibility=False):
        if checkVisibility:
            self.is_present_and_visible()
        self.driver.find_element(self.locator.by, self.locator.value).click()
        return self
